# -*- project-name: VASM -*-
package VASM::Action;

use strict;
use warnings;
use XML::Parser;
use XML::Writer;

our $VERSION = '1.02';
my $indent = 2; # The Indent Level of the Gods

sub new {
  my ($self, $file) = @_;

  my $instance = []; # Commands pushed and popped using arrayref
  bless $instance, $self;

  # Initialize the instance if a file is specified
  $instance->Parse($file) if defined $file;

  return $instance;
}

# Add another command string to the end of the actions list
sub Push {
  my ($self, $command) = @_;

  push @{ $self }, $command;
  return;
}

# Remove a command string from the end of the actions list
sub Pop {
  my ($self) = @_;
  
  pop @{ $self };
  return;
}

# It is expected that each string in the actions catalog will receive the same
# arguments, in the same order. It is not the job of Render to check the
# veracity of command lines it generates, just to produce them.
sub Render {
  my ($self, @args) = @_;

  # Render each command string, assuming some exist!
  return map { sprintf $_, @args } @{ $self }
    if scalar @{ $self };

  return;
}

# Just return the contents of the instance
sub Dump {
  my ($self) = @_;
  
  return @{ $self };
}

sub Parse {
  my ($self, $file) = @_;
  # If this fails, XML::Parser will emit an error message and end the program.
  # Unfortunately, this message is always English. :( There is at least an
  # error context, which can be universally understood by the programmer.
  my $parser = XML::Parser->new(Style => 'Tree', ErrorContext => 3);
  my $tree = $parser->parsefile($file); my $tags = $tree->[1];
  
  # Iterate through tags under <actions>
  for my $index (grep { $_ % 2 } (0..$#{ $tags })) {
    next if $tags->[$index] eq '0'; # Skip text elements 

    # Otherwise, it /has to be/ a command string. Return if it does not
    # contain a text string or holds no content
    return unless $tags->[$index + 1]->[1] eq '0'
      and defined $tags->[$index + 1]->[2];

    # Push the new member then
    $self->Push($tags->[$index + 1]->[2]);
  }

  return 1; # Success!
}

# Write out the contents of the instance to a file, given a filehandle
sub Write {
  my ($self, $fh) = @_;

  # Instantiate an IO::Handle object representing the specified XML file
  my $handle = IO::Handle->new_from_fd(fileno($fh), 'w');
  return unless defined $handle; # Return unless successful

  # Now create an XML::Writer instance and turn that puppy out
  my $xml = XML::Writer->new(OUTPUT => $handle, ENCODING => 'utf-8',
                             DATA_MODE => 1, DATA_INDENT => $indent);

  # Open up the document
  $xml->xmlDecl;
  $xml->startTag('actions');
  $xml->comment('Generated by VASM; hanumizzle was here'); # Had to... :D

  # Emit the 'command' tags
  for my $command (@{ $self }) {
    $xml->dataElement('command', $command);
  }

  $xml->endTag('actions'); $xml->end; # End the document...
  $handle->close; # ...close the filehandle and...done

  return 1; # Successful!
}
  
1;

__END__

=head1 NAME

VASM::Action

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 METHODS

=head1 AUTHORS

hanumizzle L<hanumizzle@gmail.com> wrote VASM::Action.

=cut

